@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: Jul  8 2021 (17:09) 
 ## Version: 
-## Last-Updated: Feb 13 2022 (23:09) 
+## Last-Updated: Jun  2 2022 (11:43) 
 ##           By: Brice Ozenne
-##     Update #: 51
+##     Update #: 125
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -20,7 +20,8 @@
 ##'
 ##' @param object a \code{lmm} object.
 ##' @param newdata [data.frame] the covariate values for each cluster.
-##' @param keep.newdata [logical] Should the argument \code{newdata} be output along side the predicted values?
+##' @param format [character] Should the predicted mean be output relative as a vector (\code{"long"}), or as a matrix with in row the clusters and in columns the outcomes (\code{"wide"}).
+##' @param keep.newdata [logical] Should the argument \code{newdata} be output along side the predicted values? The output will then be a \code{data.frame}.
 ##' @param impute [logical] Should the missing data in the outcome be imputed based on covariates and other outcome values from the same cluster.
 ##' @param se.impute [character] If \code{FALSE} the most likely value is imputed. Otherwise the imputed value is sampled from a normal distribution.
 ##' The value of the argument determine which standard deviation is used: all uncertainty about the predicted value (\code{"total"}),
@@ -28,7 +29,9 @@
 ##' Passed to \code{predict.lmm}.
 ##' @param ... Not used. For compatibility with the generic method.
 ##'
-##' @return When \code{keep.newdata==FALSE}: \itemize{
+##' @return When \code{format="wide"}, a data.frame with as many rows as clusters.
+##' When \code{format="long"} or \code{keep.newdata==TRUE}, a data.frame with as many rows as observations.
+##' Otherwise: \itemize{
 ##' \item if \code{impute=FALSE} a vector of length the number of row of newdata containing the fitted values (i.e. based on the covariates only).
 ##' \item if \code{impute=TRUE} a vector of length the number of missing values in the outcome of newdata containing the cluster-specific conditional means
 ##' (i.e. based on the covariates and outcome measurements from the same cluster).
@@ -72,8 +75,17 @@
 
 ## * fitted.lmm (code)
 ##' @export
-fitted.lmm <- function(object, newdata = NULL, keep.newdata = FALSE, impute = FALSE, se.impute = FALSE, ...){
-    
+fitted.lmm <- function(object, newdata = NULL, format = "long",
+                       keep.newdata = FALSE, impute = FALSE, se.impute = FALSE, ...){
+
+    ## ** extract from object
+    outcome.var <- object$outcome$var
+    cluster.var <- object$cluster$var
+    time.var <- object$time$var
+    object.fitted <- object$fitted
+    object.data <- object$data ## with columns XXindexXX, XXclusterXX, ... and all observations (including NAs)
+    object.data.original <- object$data.original 
+
     ## ** normalize user imput
     dots <- list(...)
     if(length(dots)>0){
@@ -82,61 +94,77 @@ fitted.lmm <- function(object, newdata = NULL, keep.newdata = FALSE, impute = FA
     if(!is.logical(impute)){
         stop("Argument \'impute\' should be TRUE or FALSE. \n")
     }
+    if(impute && "imputed" %in% names(newdata)){
+        stop("Argument \'newdata\' should not contain a column called \"imputed\". \n")
+    }
+    if(!impute){
+        se.impute <- FALSE
+    }
+    type.prediction <- ifelse(impute,"dynamic", "static")
 
-    ## ** impute missing values
-    if(impute){
-        if(is.null(newdata)){
-            newdata <- object$data.original
-        }
-        e.impute <- stats::predict(object, newdata = newdata, type = "dynamic", se = se.impute, keep.newdata = TRUE)
-        index.NA <- which(!is.na(e.impute$estimate))
-        if(length(index.NA) > 0 && "se" %in% names(e.impute) == FALSE){
-            newdata[index.NA,object$outcome$var] <- e.impute[index.NA,"estimate"]
-        }else{
-            newdata[index.NA,object$outcome$var] <- stats::rnorm(length(index.NA), mean = e.impute[index.NA,"estimate"], sd = e.impute[index.NA,"se"])
-        }
+    format <- match.arg(format, c("wide","long"))
+    
+    test.original.data <- is.null(newdata)
+    if(test.original.data){
+        newdata <- object.data.original
+    }
 
-        if(keep.newdata==FALSE){
-            return(newdata[index.NA,object$outcome$var])
-        }else{
-            if("imputed" %in% names(newdata)){
-                stop("Argument \'newdata\' should not contain a column called \"imputed\". \n")
-            }
-            newdata$imputed <- FALSE
-            newdata$imputed[index.NA] <- TRUE
-        }
+    if(format == "wide") {
+        keep.newdata <- TRUE
     }
 
     ## ** compute predictions
-    ## design matrix
-    if(is.null(newdata)){
-        newdata <- object$data.original
-            
-        if(length(object$index.na)>0){
-            prediction <- rep(NA, NROW(newdata))
-            prediction[-object$index.na] <- object$fitted
+    e.pred <- stats::predict(object,
+                             newdata = newdata,
+                             type = type.prediction,
+                             se = se.impute,
+                             keep.newdata = keep.newdata)
+    
+    ## ** store
+    if(impute){ ## store dynamic predictions
+
+            index.NA <- which(!is.na(e.pred$estimate))
+            if(keep.newdata == FALSE){
+                if("se" %in% names(e.pred) == FALSE){
+                    out <- e.pred[index.NA,"estimate"]
+                }else{
+                    out <- stats::rnorm(length(index.NA),
+                                        mean = e.pred[index.NA,"estimate"],
+                                        sd = e.pred[index.NA,"se"])
+                }
+                
+            }else if(length(index.NA) > 0){
+                if("se" %in% names(e.pred) == FALSE){
+                    newdata[index.NA,outcome.var] <- e.pred[index.NA,"estimate"]
+                }else{
+                    newdata[index.NA,outcome.var] <- stats::rnorm(length(index.NA),
+                                                                  mean = e.pred[index.NA,"estimate"],
+                                                                  sd = e.pred[index.NA,"se"])
+                }
+                newdata$imputed <- FALSE
+                newdata$imputed[index.NA] <- TRUE
+                out <- newdata
+            }
+            value.var <- c(outcome.var,"imputed")
+    }else{ ## store static predictions
+
+        if(keep.newdata == FALSE){
+            out <- e.pred[["estimate"]]
         }else{
-            prediction <- as.vector(object$fitted)
+            out <- e.pred
         }
-    }else{
-        ## extract coefficients
-        beta <- stats::coef(object, effects = "mean")
-        name.beta <- names(beta)
+        value.var <- "estimate"
 
-        ## generate design matrix
-        X.beta <- stats::model.matrix(object, data = newdata, effects = "mean")
-
-        ## compute predictions
-        prediction <- as.vector(X.beta %*% beta)
     }
 
+    ## ** convert to wide format
+    if(format=="wide"){
+        out <- stats::reshape(data = out[,c(attr(time.var,"original"), attr(cluster.var,"original"), value.var),drop=FALSE], 
+                              direction = "wide", timevar = attr(time.var,"original"), idvar = attr(cluster.var,"original"), v.names = value.var)
+    }
 
     ## ** export
-    if(keep.newdata){
-        return(cbind(newdata, estimate = prediction))
-    }else{
-        return(prediction)
-    }
+    return(out)
     
 }
 
