@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: mar  5 2021 (21:28) 
 ## Version: 
-## Last-Updated: Feb 13 2022 (23:12) 
+## Last-Updated: maj 27 2022 (13:13) 
 ##           By: Brice Ozenne
-##     Update #: 483
+##     Update #: 487
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -116,12 +116,12 @@ vcov.lmm <- function(object, effects = "mean", robust = FALSE, df = FALSE, strat
                 if(any(duplicated(names(p)))){
                     stop("Incorrect argument \'p\': contain duplicated names \"",paste(unique(names(p)[duplicated(names(p))]), collapse = "\" \""),"\".\n")
                 }
-                if(any(names(object$param$type) %in% names(p) == FALSE)){
-                    stop("Incorrect argument \'p\': missing parameter(s) \"",paste(names(object$param$type)[names(object$param$type) %in% names(p) == FALSE], collapse = "\" \""),"\".\n")
+                if(any(names(object$param) %in% names(p) == FALSE)){
+                    stop("Incorrect argument \'p\': missing parameter(s) \"",paste(names(object$param)[names(object$param) %in% names(p) == FALSE], collapse = "\" \""),"\".\n")
                 }
-                p <- p[names(object$param$value)]
+                p <- p[names(object$param)]
             }else{
-                p <- object$param$value
+                p <- object$param
             }
 
             outMoments <- .moments.lmm(value = p, design = design, time = object$time, method.fit = object$method.fit, type.information = type.information,
